﻿#ifndef __SM2_HEADER__
#define __SM2_HEADER__

#include "sm3.h"

#ifdef __cplusplus
extern "C"{
#endif

#define SM2KEYSIZE  64
#define SIGSIZE     64
#define IDMAXSIZE   64
#define RANDOMSIZE  32

#define SM3_E nettle_SM3_E
#define SM3_256_z nettle_SM3_256_z
#define sm2_keygen nettle_sm2_keygen
#define sm2_sign nettle_sm2_sign
#define sm2_verify nettle_sm2_verify
#define sm2_encrypt nettle_sm2_encrypt
#define sm2_decrypt nettle_sm2_decrypt
#define sm2_sponsor_keyagreement_init nettle_sm2_sponsor_keyagreement_init
#define sm2_responder_keyagreement_calculate nettle_sm2_responder_keyagreement_calculate
#define sm2_sponsor_keyagreement_calculate nettle_sm2_sponsor_keyagreement_calculate
#define sm2_keyagreement_check nettle_sm2_keyagreement_check




/*SM2公钥*/
typedef struct PubKey_st{
    unsigned char x[SM2KEYSIZE];
    unsigned char y[SM2KEYSIZE];
    int xLen;
    int yLen;
}sm2_pubkey;

/*SM2私钥*/
typedef struct PrivKey_st{
    unsigned char key[SM2KEYSIZE];
    int len;
}sm2_privkey;

typedef struct Sm2Sig_st{
    unsigned char cr[SIGSIZE];
    unsigned char cs[SIGSIZE];
    int crLen;
    int csLen;
}Sm2Sig;


/*ECC密钥协商参数结构*/
typedef struct AgreementData_st {
	sm2_privkey  *privkey;                  //协商私钥，由密钥句柄获得
	sm2_pubkey   *pubkey;                   //协商公钥，由密钥句柄获得
	sm2_pubkey   *temppubkey;               //协商的临时公钥
	unsigned char userid[IDMAXSIZE];    //用户id
	unsigned int useridlen;             //用户id的长度
	unsigned char randmo[RANDOMSIZE];   //发起方产生的随机数
	unsigned int  randmolen;            //发起方产生的随机数长度
	unsigned int keybits;               //协商密钥的长度
}AgreementData;


/**
 * @brief 计算SM3 E值
 * 
 * @param userID 用户ID
 * @param userIDLen 用户ID长度
 * @param pubKey 公钥
 * @param msg  有效数据
 * @param msgLen 有效数据长度
 * @param digest 计算的SM3杂凑值
 * @return 0， 成功
 *         -1，失败
 */
int SM3_E(unsigned char *userID, int userIDLen, sm2_pubkey* pubKey, unsigned char *msg, int msgLen, unsigned char *hash);


/**
 * @brief 生成密钥对
 * 
 * @param pubKey 公钥
 * @param privKey 私钥
 * @return 0， 成功
 *         -1， 失败
 */
int sm2_keygen(sm2_pubkey* pubKey, sm2_privkey* privKey, void *random_ctx,nettle_random_func *random);


/**
 * @brief 计算签名值
 * 
 * @param digest 杂凑值
 * @param digestlen 杂凑值长度
 * @param privKey 私钥
 * @param sign 签名值
 * @return 0， 成功
 *         -1， 失败
 */
int sm2_sign(unsigned char *digest, int digestlen, sm2_privkey* privKey, Sm2Sig* sign, void *random_ctx,nettle_random_func *random);


/**
 * @brief 验证签名值
 * 
 * @param digest 杂凑值
 * @param digestlen 杂凑值长度
 * 
 * @param sign 签名值
 * @param pubKey 公钥
 * @return 0， 验证成功
 *         -1，验证失败
 */
int sm2_verify(unsigned char *digest,int digestlen, Sm2Sig* sign, sm2_pubkey* pubKey);


/**
 * @brief 数据加密
 * 
 * @param pubKey 公钥
 * @param inputData 输入数据
 * @param inputLength 输入数据长度
 * @param outData  密文数据
 * @param outputLength 密文长度
 * @return 0， 成功
 *         -1, 失败
 */
int sm2_encrypt(const sm2_pubkey* pubKey, const unsigned char *inputData,  int inputLength, unsigned char *outData, int *outputLength,void *random_ctx,nettle_random_func *random);


/**
 * @brief 数据解密
 * 
 * @param privKey 私钥
 * @param inputData  输入数据
 * @param inputLength 输入数据长度
 * @param outData 解密结果数据
 * @param outputLength 解密结果数据长度
 * @return 0，成功
 *         -1， 失败
 */
int sm2_decrypt(const sm2_privkey* privKey, const unsigned char *inputData, int inputLength, unsigned char *outData, int *outputLength);

/**
 * @brief 发起方密钥协商初始化
 * 
 * @param pubKey 公钥
 * @param random 随机值
 * @param randomLen 随机值
 * @return 0， 成功
 *         -1，失败
 */
int sm2_sponsor_keyagreement_init(sm2_pubkey* pubKey, unsigned char *random,  int *randomLen, void *random_ctx,nettle_random_func *Random);


/**
 * @brief 响应方计算密钥协商值
 * 
 * @param sponsorTmpPubKey 发起方临时公钥
 * @param sponsorPubKey 发起方公钥
 * @param responsePrivKey 响应方私钥
 * @param responsePubKey 响应方公钥
 * @param sponsorId   发起方ID
 * @param sponsorIdLen 发起方ID长度
 * @param responseId 响应方ID
 * @param responseIdLen 响应方ID长度
 * @param keyLen 密钥长度
 * @param keyBuf 密钥
 * @param responseTmpPubKey 响应方临时公钥
 * @param tmpXv  临时数据
 * @param xvlen 临时数据长度
 * @param tmpYv  临时数据
 * @param yvlen 临时数据长度
 * @param responseHash 响应方生成的Hash值
 * @return 0， 成功
 *         -1， 失败 
 */
int sm2_responder_keyagreement_calculate(sm2_pubkey* sponsorTmpPubKey, sm2_pubkey* sponsorPubKey, sm2_privkey* responsePrivKey, sm2_pubkey* responsePubKey,
	                           unsigned char *sponsorId, int sponsorIdLen, unsigned char *responseId, int responseIdLen,
	                           unsigned int  keyLen, unsigned char *keyBuf, sm2_pubkey* responseTmpPubKey,
	                           unsigned char *tmpXv,  int *xvlen, unsigned char *tmpYv,  int *yvlen,unsigned char *responseHash, void *random_ctx,nettle_random_func *random);


/**
 * @brief 发起方计算密钥协商值
 * 
 * @param sponsorTmpPubKey 发起方临时公钥
 * @param sponsorPubKey 发起方公钥
 * @param sponsorPrivKey 发起方私钥
 * @param responsePubKey 响应方公钥
 * @param sponsorId 发起方ID
 * @param sponsorIdLen 发起方ID长度
 * @param responseId 响应方ID
 * @param responseIdLen 响应方ID长度
 * @param responseTmpPubKey 响应方临时公钥
 * @param sponsorRandom 发起方随机值
 * @param sponsorRandomLen 发起方随机值长度
 * @param keyLen 密钥长度
 * @param keyBuf 密钥
 * @param sponsorTmpHash 发起方生成的hash 值
 * @param sponsorHash 响应方生成的hash 值
 * @return 0， 成功
 *         -1， 失败
 */
int sm2_sponsor_keyagreement_calculate( sm2_pubkey* sponsorTmpPubKey, sm2_pubkey* sponsorPubKey, sm2_privkey* sponsorPrivKey,  sm2_pubkey* responsePubKey,
                            unsigned char *sponsorId, int sponsorIdLen, unsigned char *responseId, int responseIdLen,
                            sm2_pubkey* responseTmpPubKey, unsigned char *sponsorRandom,  int sponsorRandomLen,
                            unsigned int  keyLen, unsigned char *keyBuf,
                            unsigned char *sponsorTmpHash,  unsigned char *sponsorHash);

/**
 * @brief 响应方验证协商
 * 
 * @param sponsorPubKey  发起方公钥
 * @param responsePubKey 响应方公钥
 * @param sponsorTmpPubKey 发起方临时公钥
 * @param responseTmpPubKey 响应方临时公钥
 * @param tmpXv 响应方临时数据xv
 * @param xvlen 响应方临时数据xv长度
 * @param tmpYv 响应方临时数据yv
 * @param yvlen 响应方临时数据yv长度
 * @param sponsorId 发起方ID
 * @param sponsorIdLen 发起方ID长度
 * @param responseId 响应方ID
 * @param responseIdLen 响应方ID长度
 * @param reponseHash 计算的hash值
 * @return 0， 成功
 *         -1， 失败
 */
int sm2_keyagreement_check(sm2_pubkey* sponsorPubKey, sm2_pubkey* responsePubKey, sm2_pubkey* sponsorTmpPubKey, sm2_pubkey* responseTmpPubKey,
                            unsigned char *tmpXv, int xvlen,unsigned char *tmpYv, int yvlen,
                            unsigned char *sponsorId, int sponsorIdLen,unsigned char *responseId, int responseIdLen, unsigned char *reponseHash);



#ifdef __cplusplus
}
#endif


#endif
